@@ -274,347 +274,13 @@ as_yamlet.character <- function(x, default_keys = getOption('yamlet_default_keys
   as_yamlet(as_yam(x, ...), default_keys = default_keys, ...)
 }
 
-#' Redecorate a List-like Object
-#'
-#' Redecorates a list-like object.
-#' Equivalent to \code{decorate( ..., overwrite = TRUE)}.
-
-#'
-#' @param x object
-#' @param ... passed arguments
-#' @param overwrite passed to \code{\link{decorate}}
-#' @export
-#' @family decorate
-#' @return a list-like object, typically data.frame
-#' @examples
-#' library(dplyr)
-#' library(magrittr)
-#' library(csv)
-#' file <- system.file(package = 'yamlet', 'extdata','quinidine.csv')
-#' x <- decorate(as.csv(file))
-#' x %>% select(Subject) %>% as_yamlet
-#' x %<>% redecorate('Subject: Patient Identifier')
-#' x %>% select(Subject) %>% as_yamlet
-#'
-redecorate <- function(x, ..., overwrite = TRUE)decorate(x, ..., overwrite = overwrite)
-
-#' Decorate a List-like Object
-#'
-#' Decorates a list-like object. Generic.
-#' See \code{\link{decorate.character}}.
-
-#'
-#' @param x object
-#' @param ... passed arguments
-#' @export
-#' @keywords internal
-#' @family decorate
-#' @return a list-like object, typically data.frame
-#' @examples
-#' library(csv)
-#' file <- system.file(package = 'yamlet', 'extdata','quinidine.csv')
-#' x <- decorate(as.csv(file))
-#' identical(decorate(as.csv(file)), decorate(file))
-#' decorations(x) # but prefer as_yamlet(x)
-#'
-#'
-decorate <- function(x,...)UseMethod('decorate')
-
-#' Decorate Character
-#'
-#' Treats \code{x} as a file path. By default,
-#' metadata is sought from a file with the same
-#' base but the 'yaml' extension.
-
-#'
-#' @param x file path for table data
-#' @param meta file path for corresponding yamlet metadata, or a yamlet object
-#' @param read function or function name for reading x
-#' @param ext file extension for metadata file, if relevant
-# @param coerce whether to coerce to factor where guide has length > 1
-#' @param ... passed to read (if accepted) and to \code{\link{as_yamlet.character}}
-#' @return class 'decorated' 'data.frame'
-#' @importFrom csv as.csv
-#' @export
-#' @family decorate
-#' @family interface
-#' @examples
-#' file <- system.file(package = 'yamlet', 'extdata','quinidine.csv')
-#' meta <- system.file(package = 'yamlet', 'extdata','quinidine.yaml')
-#' identical(
-#'   decorate(file),
-#'   decorate(file, meta = meta)
-#' )
-#' identical(
-#'   decorate(file, meta = as_yamlet(meta)),
-#'   decorate(file, meta = meta)
-#' )
-#' a <- decorate(file)
-#' b <- resolve(decorate(file))
-#' c <- resolve(decorate(
-#'   file,
-#'   read = read.table,
-#'   quote = "",
-#'   as.is = FALSE,
-#'   sep = ',',
-#'   header = TRUE,
-#'   na.strings = c('', '\\s', '.','NA'),
-#'   strip.white = TRUE,
-#'   check.names = FALSE
-#' ))
-#' d <- decorate(
-#'   file,
-#'   read = read.table,
-#'   quote = "",
-#'   as.is = FALSE,
-#'   sep = ',',
-#'   header = TRUE,
-#'   na.strings = c('', '\\s', '.','NA'),
-#'   strip.white = TRUE,
-#'   check.names = FALSE
-#' )
-#'
-#' # Importantly, b and c are identical with respect to factors
-#' cbind(
-#'   `as.is/!resolve`   = sapply(a, class), # no factors
-#'   `as.is/resolve`    = sapply(b, class), # factors made during decoration
-#'   `!as.is/resolve`   = sapply(c, class), # factors made twice!
-#'   `!as.is/!resolve`  = sapply(d, class)  # factors made during read
-#' )
-#' str(a$Smoke)
-#' str(b$Smoke)
-#' str(c$Smoke)
-#' str(d$Smoke)
-#' levels(c$Creatinine)
-#' levels(d$Creatinine) # level detail retained as 'guide'
-
-decorate.character <- function(
-  x,
-  meta = NULL,
-  read  = getOption('yamlet_import', as.csv),
-  ext  = getOption('yamlet_extension', '.yaml'),
-  # coerce = getOption('yamlet_coerce',FALSE),
-  ...
-){
-  stopifnot(length(x) == 1)
-  if(!file.exists(x))stop('could not find file ', x)
-  read <- match.fun(read)
-  args <- list(...)
-  # args <- args[names(args) %in% names(formals(read))] # debilitating
-  args <- c(list(x),args)
-  y <- do.call(read, args)
-  if(is.null(meta)){
-    meta <- sub('\\.[^.]*$','',x) # remove last dot and any trailing chars
-    meta <- paste0(meta, ext)
-  }
-  if(is.character(meta) & length(meta) == 1){
-    meta <- try(as_yamlet(meta,...))
-  }
-  if(class(meta) != 'yamlet') stop('could not interpret meta: ', meta)
-  decorate(
-    y,
-    meta = meta,
-    # coerce = coerce,
-    ...
-  )
-}
-
-#' Decorate List
-#'
-#' Decorates a list-like object. Takes metadata
-#' in yamlet format and loads it onto corresponding
-#' list elements as attributes.
-#'
-#'
-#' @param x object inheriting from \code{list}
-#' @param meta file path for corresponding yaml metadata, or a yamlet or something coercible to yamlet; an attempt will be made to guess the file path if x has a 'source' attribute (as for \code{\link[csv]{as.csv}})
-#' @param ext file extension for metadata file, if relevant
-# @param coerce whether to coerce to factor where guide has length > 1
-#' @param overwrite whether to overwrite attributes that are already present (else give warning)
-#' @param ... passed to \code{\link{as_yamlet.character}} (by method dispatch)
-#' @return like x but with 'decorated' as first class element
-#' @export
-#' @keywords internal
-#' @family decorate
-#' @examples
-#' example(decorate.data.frame)
-#'
-decorate.list <- function(
-  x,
-  meta = NULL,
-  ext = getOption('yamlet_extension', '.yaml'),
-  #coerce = getOption('yamlet_coerce', FALSE),
-  overwrite = getOption('yamlet_overwrite', FALSE),
-  ...
-){
-  if(is.null(meta)) meta <- attr(x, 'source')
-  if(is.null(meta)) stop('could not guess metadata location; supply meta')
-  m <- try(silent = TRUE, as_yamlet(meta))
-  if(inherits(m, 'yamlet')) meta <- m
-  if(is.character(meta) & length(meta) == 1){
-    meta <- sub('\\.[^.]*$','',meta) # remove last dot and any trailing chars
-    meta <- paste0(meta, ext)
-    meta <- try(as_yamlet(meta, ...))
-  }
-  if(class(meta) != 'yamlet') stop('could not interpret meta: ', meta)
-
-  for(item in names(x)){ # if list has no names, nothing happens
-    if(item %in% names(meta)){ # if list has names, name '' should not be reached
-      val <- meta[[item]]
-      for(attrb in names(val)){
-        if(attrb == ''){ # warn if name is ''
-          warning('ignoring anonymous attribute for ', item)
-          next
-        }
-        if(attrb %in% names(attributes(x[[item]]))){
-          if(!overwrite){
-            warning('not overwriting ', attrb, ' attribute of ', item)
-            next
-          }
-        }
-        attr(x[[item]], attrb) <- val[[attrb]]
-      }
-    }
-  }
-  class(x) <- union('decorated', class(x))
-  x
-}
-
-
-#' Decorate Data Frame
-#'
-#' Decorates a data.frame. Expects metadata in yamlet
-#' format, and loads it onto columns as attributes.
-
-#'
-#' @param x data.frame
-#' @param meta file path for corresponding yaml metadata, or a yamlet; an attempt will be made to guess the file path if x has a 'source' attribute
-# @param coerce whether to coerce to factor where guide is a list
-#' @param ... passed to \code{\link{decorate.list}}
-#' @return class 'decorated' 'data.frame'
-#' @export
-#' @family interface
-#' @family decorate
-#' @seealso decorate.list
-#' @examples
-#' library(csv)
-#' file <- system.file(package = 'yamlet', 'extdata','quinidine.csv')
-#' meta <- system.file(package = 'yamlet', 'extdata','quinidine.yaml')
-#' a <- decorate(as.csv(file))
-#' b <- decorate(as.csv(file), meta = as_yamlet(meta))
-#' c <- decorate(as.csv(file), meta = meta)
-#' d <- decorate(as.csv(file), meta = file)
-#' e <- resolve(decorate(as.csv(file)))
-#'
-#' # Most import methods are equivalent.
-#' identical(a, b)
-#' identical(a, c)
-#' identical(a, d)
-#' identical(a, e)
-decorate.data.frame <- function(
-  x,
-  meta = NULL,
-  # coerce = getOption('yamlet_coerce',FALSE),
-  ...
-)decorate.list(
-  x,
-  meta = meta,
-  #coerce = coerce,
-  ...
-)
-
-#' Retrieve Decorations
-#'
-#' Retrieve the decorations of something.
-#' Generic, with method for data.frame.
-#'
-#' @param x object
-#' @param ... passed arguments
-#' @export
-#' @keywords internal
-#' @family decorate
-#' @return see methods
-#' @examples
-#' library(csv)
-#' file <- system.file(package = 'yamlet', 'extdata','quinidine.csv')
-#' x <- decorate(as.csv(file))
-#' decorations(x) # prefer as_yamlet(x)
-
-decorations <- function(x,...)UseMethod('decorations')
-
-#' Retrieve Decorations for Data Frame
-#'
-#' Retrieve the decorations of a data.frame; i.e., the metadata
-#' used to decorate it. Returns a list with same names as the data.frame.
-#' By default, \code{class} attributes are excluded from the result,
-#' as this is an attribute you likely don't want to manipulate independently.
-#' Consider carefully whether the default handling of factor levels
-#' (see \code{coerce} argument) is appropriate for your application.
-
-#'
-#' @param x data.frame
-#' @param coerce logical: whether to coerce factor levels to guide; alternatively, a key for the levels
-#' @param exclude_attr attributes to remove from the result
-#' @param ... ignored
-#' @export
-#' @family decorate
-#' @return named list
-#' @examples
-#' library(csv)
-#' library(magrittr)
-#' file <- system.file(package = 'yamlet', 'extdata','quinidine.csv')
-#' x <- decorate(as.csv(file))[,c('conc','Race')]
-#' y <- decorate(as.csv(file))[,c('conc','Race')] %>% resolve
-#' decorations(x)
-#' decorations(y)
-#' decorations(y, coerce = TRUE)
-#' decorations(y, coerce = 'codelist')
-#' decorations(y, exclude_attr = NULL)
-
-decorations.data.frame <- function(
-  x,
-  coerce = getOption('yamlet_coerce_decorations', FALSE),
-  exclude_attr = getOption('yamlet_exclude_attr', 'class'),
-  ...
-){
-  stopifnot(length(exclude_attr) == 0 || is.character(exclude_attr))
-  out <- lapply(x, attributes)
-  levs_key <- 'guide'
-  if(!is.logical(coerce)){
-    if(is.character(coerce))
-      if(length(coerce) == 1){
-        levs_key <- coerce
-        coerce <- TRUE
-      }
-  }
-  if(!is.logical(coerce)){
-    warning('coerce value not logical')
-  }else{
-    if(coerce){
-      for(i in seq_along(out)){
-        if('class' %in% names(out[[i]])){
-          if(any(out[[i]]$class == 'factor')){ # factor or ordered factor
-            out[[i]]$class <- NULL
-            names(out[[i]])[names(out[[i]]) == 'levels'] <- levs_key
-          }
-        }
-      }
-    }
-  }
-  for(i in exclude_attr){
-    for(j in names(out)){
-      if(i %in% names(out[[j]])) out[[j]][[i]] <- NULL
-    }
-  }
-  out
-}
 
 #' Coerce Data Frame to Yamlet
 #'
 #' Coerces data.frame to yamlet. Assigns class 'yamlet' to a data.frame's decorations.
 #'
 #' @param x data.frame
-#' @param... passed arguments
+#' @param ... passed to \code{\link{decorations}}
 #' @family as_yamlet
 #' @export
 #' @keywords internal
@@ -629,12 +295,31 @@ as_yamlet.data.frame <- function(x, ...){
   class(out) <- 'yamlet'
   out
 }
+#' Coerce List to Yamlet
+#'
+#' Coerces list to yamlet. Assigns class 'yamlet'.
+#' Checks that list has names.
+#'
+#' @param x list
+#' @param ... ignored
+#' @family as_yamlet
+#' @export
+#' @keywords internal
+#' @return yamlet
+#' @examples
+#' x <- list(a = 2, b = 3)
+#' as_yamlet(x)
+as_yamlet.list <- function(x, ...){
+  if(is.null(names(x)))stop('list must have names for conversion to yamlet')
+  class(x) <- 'yamlet'
+  x
+}
 #' Coerce Yamlet to Yamlet
 #'
 #' Coerces yamlet to yamlet. Currently a non-operation.
 #'
 #' @param x yamlet
-#' @param... ignored
+#' @param ... ignored
 #' @family as_yamlet
 #' @export
 #' @keywords internal
@@ -656,8 +341,8 @@ as_yamlet.yamlet<- function(x, ...)x
 #' attribute of the result.
 #'
 #' @param x yamlet
-#' @param default_keys names that may be omitted in left subsets
 #' @param ... ignored
+#' @param default_keys names that may be omitted in left subsets
 #' @export
 #' @keywords internal
 #' @return yam
@@ -668,7 +353,11 @@ as_yamlet.yamlet<- function(x, ...)x
 #' as_yam(as_yamlet(c('amt: [ dose, mg ]')))
 #' as_yam(as_yamlet(c('amt: [ guide: mg, label: dose ]')))
 #'
-as_yam.yamlet <- function(x, default_keys = getOption('yamlet_default_keys', list('label','guide')), ...){
+as_yam.yamlet <- function(
+  x,
+  ...,
+  default_keys = getOption('yamlet_default_keys', list('label','guide'))
+){
   default_keys <- as.list(default_keys)
   for(nm in names(x)){
     candidates <- unlist(default_keys)
@@ -946,17 +635,18 @@ list2encoding <- function(x, ...){
 #' is supplied, conditional guides will be ignored.
 #'
 #' @param x yamlet
+#' @param ... ignored
 #' @param target attribute to encode
 #' @param data optional data.frame for guide context
-#' @param ... ignored
 #' @return yamlet, with guide elements possibly transformed to encodings
 #' @export
+#' @keywords internal
 #' @family encode
 #' @examples
 #' meta <- system.file(package = 'yamlet', 'extdata','quinidine.yaml')
 #' meta <- as_yamlet(meta)
 #' meta <- encode(meta)
-encode.yamlet <- function(x, target = 'guide', data = NULL, ...){
+encode.yamlet <- function(x, ..., target = 'guide', data = NULL){
   for(i in seq_along(x)){
     t <- x[[i]][[target]]
     if(!is.null(t)){
@@ -1056,8 +746,8 @@ print.yamlet <- function(x, ...){
 #' but also reads text fragments.
 #'
 #' @param x file path for yamlet, or vector of yamlet in storage syntax
-#' @param default_keys character: default keys for the first n anonymous members of each element
 #' @param ... passed to \code{\link{as_yamlet}}
+#' @param default_keys character: default keys for the first n anonymous members of each element
 #' @export
 #' @family interface
 #' @seealso \code{\link{decorate.data.frame}}
@@ -1072,11 +762,11 @@ print.yamlet <- function(x, ...){
 #' identical(x, decorate(file))
 read_yamlet <- function(
   x,
+  ...,
   default_keys = getOption(
     'yamlet_default_keys',
     list('label','guide')
-  ),
-  ...
+  )
 ){
   stopifnot(is.character(x))
   as_yamlet(x, default_keys = default_keys, ...)
@@ -1087,12 +777,12 @@ read_yamlet <- function(
 #' but returns invisible storage format instead of invisible storage location.
 #'
 #' @param x something that can be coerced to class 'yamlet', like a yamlet object or a decorated data.frame
+#' @param ... passed to \code{\link{as_yamlet}}
 #' @param con passed to \code{\link{writeLines}}
 #' @param eol end-of-line; passed to \code{\link{writeLines}} as \code{sep}
 #' @param useBytes passed to \code{\link{writeLines}}
 #' @param default_keys character: default keys for the first n anonymous members of each element
 #' @param fileEncoding if \code{con} is character, passed to \code{\link{file}} as \code{encoding}
-#' @param ... passed to \code{\link{as_yamlet}}
 #' @export
 #' @family interface
 #' @seealso \code{\link{decorate.list}}
